define([
    "../app/Router",
    "../model/classes/surveys",
    "../model/widgets/QuestionListView",
    "../model/widgets/TabbedQuestionBrowser",
    "./_ObjectPage",
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/event",
    "dojo/_base/lang",
    "dojo/when",
    "require",
    "dojo/text!./templates/survey.html"
], function(Router, surveys, QuestionListView, TabbedQuestionBrowser, _ObjectPage, array, declare, event, lang, when, require, template) {
    return declare([_ObjectPage],{
        contextRequire: require,
        templateString: template,
        classStore: surveys,
        questionList: null,
        startup: function() {
            if ( this._started ) { return; }
            this.inherited(arguments);
            this._setupQuestionBrowser();
            this._setupListView();
            this._load();
        },
        _setupQuestionBrowser: function() {
            this.questionBrowser = new TabbedQuestionBrowser({
                region: 'center',
                'class': 'blue',
                include: 'published',
                selectedActions: {
                    "Include": {
                        callback: lang.hitch(this,this._includeQuestion),
                        icon: "Accept",
                        description: "Include in survey"
                    }
                },
                itemActions: {
                    "Info": {
                        callback: function(item){
                            if ( item.description ) { alert(item.description); }
                        },
                        icon: "Inspect",
                        description: "Show item description"
                    }
                }
            },this.questionBrowser);
            this.questionBrowser.startup();
        },
        _setupListView: function() {
            this.questionList = new QuestionListView({
                region: 'center',
                name: 'questions'
            },this.surveyListViewNode);
            this.questionList.startup();
        },
        _refresh: function() {
            this.titleNode.innerHTML = this.object.title || "(set title in properties)";
            this.propertiesDialog.set('value',{survey:this.object});
            this.questionList.set('value',
                                  this.object.questions);
        },
        _save: function() {
            this.object.questions = this.questionList.get('value');
            return this.inherited(arguments);
        },
        _includeQuestion: function(question) {
            this.questionList.appendItem(question);
        },
        _onShowProperties: function(evt) {
            this.propertiesDialog.show();
            if ( evt ) { event.stop(evt); }
            return false;
        },
        _onPropertiesOk: function(evt) {
            this.propertiesDialog.hide();
            lang.mixin(this.object, this.propertiesDialog.get('value').survey);
            this.markDirty();
            this._refresh();
            if ( evt ) { event.stop(evt); }
            return false;
        },
        _onPropertiesCancel: function(evt) {
            this.propertiesDialog.hide();
            this.propertiesDialog.set('value',{survey:this.object});
            if ( evt ) { event.stop(evt); }
            return false;
        },
        _onSave: function(evt) {
            this._save();
            if ( evt ) { event.stop(evt); }
            return false;
        },
        _onSaveAndClose: function(evt) {
            this._save()
            .then(function() { 
                Router.go(surveys.getCollectionPath());
            });
            event.stop(evt);
            return false;
        },
        _onDiscardAndClose: function(evt) {
            this.markClean();
            Router.go(surveys.getCollectionPath());
        },
        _onShowPreview: function() {
            Router.go(surveys.getPreviewPath(this.object),{
                preview: true
            });
        }
    });
});

